#pragma NOIV                    // Do not generate interrupt vectors
//-----------------------------------------------------------------------------
//   File:      slave.c
//   Contents:  Hooks required to implement USB peripheral function.
//              Code written for FX2 REVE 56-pin and above.
//              This firmware is used to demonstrate FX2 Slave FIF
//              operation.
//              operation.
//   Copyright (c) 2003 Cypress Semiconductor All rights reserved
//-----------------------------------------------------------------------------
#include "fx2.h"
#include "fx2regs.h"
#include "fx2sdly.h"            // SYNCDELAY macro

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings


BOOL g_Done_frm_fpga = FALSE;

BOOL g_bTDPollInit = FALSE;
int g_nLoopCnt = 0;
BOOL g_bLedFlag = FALSE;
BOOL g_bIfconfigSetDone = FALSE;


// EZUSB FX2 PORTA = slave fifo enable(s), when IFCFG[1:0]=11
//sbit PA0 = IOA ^ 0;           // alt. func., INT0#
//sbit PA1 = IOA ^ 1;           // alt. func., INT1#
//sbit PA2 = IOA ^ 2;          	// is SLOE
//sbit PA3 = IOA ^ 3;           // alt. func., WU2
//sbit PA4 = IOA ^ 4;          	// is FIFOADR0
//sbit PA5 = IOA ^ 5;          	// is FIFOADR1
//sbit PA6 = IOA ^ 6;          	// is PKTEND
//sbit PA7 = IOA ^ 7;          	// is FLAGD

// EZUSB FX2 PORTC i/o...       port NA for 56-pin FX2
//sbit PC0 = IOC ^ 0;
//sbit PC1 = IOC ^ 1;
//sbit PC2 = IOC ^ 2;
//sbit PC3 = IOC ^ 3;
//sbit PC4 = IOC ^ 4;
//sbit PC5 = IOC ^ 5;
//sbit PC6 = IOC ^ 6;
//sbit PC7 = IOC ^ 7;

// EZUSB FX2 PORTB = FD[7:0], when IFCFG[1:0]=11
//sbit PB0 = IOB ^ 0;
//sbit PB1 = IOB ^ 1;
//sbit PB2 = IOB ^ 2;
//sbit PB3 = IOB ^ 3;
//sbit PB4 = IOB ^ 4;
//sbit PB5 = IOB ^ 5;
//sbit PB6 = IOB ^ 6;
//sbit PB7 = IOB ^ 7;

// EZUSB FX2 PORTD = FD[15:8], when IFCFG[1:0]=11 and WORDWIDE=1
//sbit PD0 = IOD ^ 0;
//sbit PD1 = IOD ^ 1;
//sbit PD2 = IOD ^ 2;
//sbit PD3 = IOD ^ 3;
//sbit PD4 = IOD ^ 4;
//sbit PD5 = IOD ^ 5;
//sbit PD6 = IOD ^ 6;
//sbit PD7 = IOD ^ 7;

//EZUSB FX2 PORTE is not bit-addressable...


void TD_Init(void)
{
	// Called once at startup
	long i = 0;
	
	CPUCS = 0x12; 					// CLKSPD[4:3]=10, for 48MHz operation, [b1:1]output CLKOUT, [b0:0]8051 Run
	SYNCDELAY;

	//-----------------------------------------------------------
	PINFLAGSAB = 0x08;				// FLAGA - EP2 Empty
	SYNCDELAY;
	
//	PINFLAGSCD = 0xE0;				// FLAGD - EP6 Full
	PINFLAGSCD = 0x0E;				// FLAGD - EP6 Full
	SYNCDELAY;

	//-----------------------------------------------------------
//	PORTACFG |= 0x80;				// b7:FLAGD
	PORTACFG |= 0x40;				//
	SYNCDELAY;

	//-----------------------------------------------------------
	IFCONFIG = 0x0B; 				// Internal clock, 48 MHz, Pin Drive, Sync, Slave FIFO interface
//	IFCONFIG = 0x43; 				// External clock, 48 MHz, Pin Tri-state, Sync, Slave FIFO interface
	SYNCDELAY;


//	REVCTL = 0x03;
//	SYNCDELAY;

	// Note: The pre-REVE EPxGPIFTCH/L register are affected, as well...
	// ...these have been replaced by GPIFTC[B3:B0] registers
	// EP4 and EP8 are not used in this implementation...

	//-----------------------------------------------------------
	EP2CFG = 0xA2;                	// out 512 bytes, 2x, bulk
	SYNCDELAY;                    

//	EP4CFG = 0xA2;                	// out 512 bytes, 2x, bulk
	EP4CFG = 0x02;                	// clear valid bit
	SYNCDELAY;                     

	EP6CFG = 0xE2;                	// in 512 bytes, 2x, bulk
	SYNCDELAY;              

//	EP8CFG = 0xE2;                	// in 512 bytes, 2x, bulk
	EP8CFG = 0x02;                	// clear valid bit
	SYNCDELAY;

	//-----------------------------------------------------------
	FIFORESET = 0x80;             	// activate NAK-ALL to avoid race conditions
	SYNCDELAY;                    	// see TRM section 15.14

	FIFORESET = 0x02;             	// reset, FIFO 2
	SYNCDELAY;                    	// 

	FIFORESET = 0x04;             	// reset, FIFO 4
	SYNCDELAY;                    	// 

	FIFORESET = 0x06;             	// reset, FIFO 6
	SYNCDELAY;                    	// 

	FIFORESET = 0x08;             	// reset, FIFO 8
	SYNCDELAY;                    	// 

	FIFORESET = 0x00;             	// deactivate NAK-ALL
	SYNCDELAY;                    	// 

	//------------------------------------------------------------
	// handle the case where we were already in AUTO mode...
	// ...for example: back to back firmware downloads...
	EP2FIFOCFG = 0x00;            	// AUTOOUT=0, WORDWIDE=0
	SYNCDELAY;                    	// 
  
	// core needs to see AUTOOUT=0 to AUTOOUT=1 switch to arm endp's
	EP2FIFOCFG = 0x11;            	// AUTOOUT=1, WORDWIDE=1
	SYNCDELAY;                    	// 

//	EP4FIFOCFG = 0x11;            	// AUTOOUT=1, WORDWIDE=1
//	SYNCDELAY;                    	// 

	EP6FIFOCFG = 0x0D;            	// AUTOIN=1, ZEROLENIN=1, WORDWIDE=1
	SYNCDELAY;                    	// 

//	EP8FIFOCFG = 0x0D;            	// AUTOIN=1, ZEROLENIN=1, WORDWIDE=1
//	SYNCDELAY;                    	// 

	//-----------------------------------------------------------
	OEA &= ~(1 << 0); 				// Declare PA.0 Input
	SYNCDELAY;

	OEA |= (1 << 1);				// Declare PA.1 Output
	SYNCDELAY;

	OEA |= (1 << 3);				// Declare PA.3 Output
	SYNCDELAY;

	IOA |= (1 << 1);				// PA.1 <= '1';		
	SYNCDELAY;

/*
	OEA |= 0x02; //Declare PA.1 as output
	SYNCDELAY;

	IOA |= 0x02; //output 1 on PA.1
	SYNCDELAY;

	OEC |= 0x01; //PC.0 as output (SYNC signal)
	SYNCDELAY;
	
	IOC |= 0x00; //output 0 on PC.0...SYNC signal is LOW 
	SYNCDELAY;
	
	OEC &= 0xFD; //PC.1 as input (Clock changing signal)
	SYNCDELAY;
*/
}

void TD_Poll(void)
{ 
	// Called repeatedly while the device is idle
	if((!g_Done_frm_fpga) && (IOA & 0x01)) 	// PA.0(IOA.0) : fpga_rdy
	{
//		IFCONFIG = 0x23;  					// External clock input, Slave FIFO interface
//		SYNCDELAY;

		IOA &= ~(1 << 1);					// PA.1 <= '0';		
		SYNCDELAY;

		g_Done_frm_fpga = TRUE;
		g_bIfconfigSetDone = TRUE;
	}

	//----------------------------------------------------------------------
	if(g_bIfconfigSetDone)
	{
		if(g_nLoopCnt > 10000)
		{
			if(!g_bLedFlag)
			{
				IOA &= ~(1 << 3);
				SYNCDELAY;

				g_bLedFlag = 1;
			}
			else
			{
				IOA |= (1 << 3);
				SYNCDELAY;
				
				g_bLedFlag = 0;
			}
			g_nLoopCnt = 0;
		}
		g_nLoopCnt ++;
	}
	
/*	
	if(!g_bTDPollInit)						// 
	{
		IOA &= ~(1 << 1);					// PA.1 <= '0';		
		SYNCDELAY;

		g_bTDPollInit = TRUE;
		g_bIfconfigSetDone = FALSE;
	}
	
	if(!(IOA & 0x01))						// if(PA.0 bit & 0x01) 	// IOA.0 : fpga usb_run
	{
		g_Done_frm_fpga = TRUE;
		g_bIfconfigSetDone = FALSE;
	}
	
	if((g_Done_frm_fpga) && (IOA & 0x01)) 	// PA.0(IOA.0) : fpga usb_run
	{
		IFCONFIG = 0x43;  					// External clock input, Slave FIFO interface
		SYNCDELAY;
		
		g_Done_frm_fpga = FALSE;
		
		g_bIfconfigSetDone = TRUE;
	}
*/
}

BOOL TD_Suspend( void )          
{ // Called before the device goes into suspend mode
	return( TRUE );
}

BOOL TD_Resume( void )          
{ // Called after the device resumes
	return( TRUE );
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------
BOOL DR_GetDescriptor( void )
{
	return( TRUE );
}

BOOL DR_SetConfiguration( void )   
{ // Called when a Set Configuration command is received
  
	if(EZUSB_HIGHSPEED())
	{ // ...FX2 in high speed mode
		EP6AUTOINLENH = 0x02;
		SYNCDELAY;
    
//		EP8AUTOINLENH = 0x02;   // set core AUTO commit len = 512 bytes
//		SYNCDELAY;
    
		EP6AUTOINLENL = 0x00;
		SYNCDELAY;

//		EP8AUTOINLENL = 0x00;
//		SYNCDELAY;
	}
	else
	{ // ...FX2 in full speed mode
		EP6AUTOINLENH = 0x00;
		SYNCDELAY;
    
//		EP8AUTOINLENH = 0x00;   // set core AUTO commit len = 64 bytes
//		SYNCDELAY;
    
		EP6AUTOINLENL = 0x40;
		SYNCDELAY;
    
//		EP8AUTOINLENL = 0x40;
//		SYNCDELAY;
	}
      
	Configuration = SETUPDAT[2];
	
	return(TRUE);        // Handled by user code
}

BOOL DR_GetConfiguration( void )   
{ // Called when a Get Configuration command is received
	EP0BUF[0] = Configuration;
	EP0BCH = 0;
	EP0BCL = 1;
   
	return(TRUE);          // Handled by user code
}

BOOL DR_SetInterface( void )       
{ // Called when a Set Interface command is received
	AlternateSetting = SETUPDAT[2];
   
	return( TRUE );        // Handled by user code
}

BOOL DR_GetInterface( void )       
{ // Called when a Set Interface command is received
	EP0BUF[0] = AlternateSetting;
	EP0BCH = 0;
	EP0BCL = 1;
   
	return( TRUE );        // Handled by user code
}

BOOL DR_GetStatus( void )
{
	return( TRUE );
}

BOOL DR_ClearFeature( void )
{
	return( TRUE );
}

BOOL DR_SetFeature( void )
{
	return( TRUE );
}

BOOL DR_VendorCmnd( void )
{
	return( TRUE );
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------
// Setup Data Available Interrupt Handler
void ISR_Sudav( void ) interrupt 0
{
	GotSUD = TRUE;         // Set flag
	EZUSB_IRQ_CLEAR( );
	USBIRQ = bmSUDAV;      // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok( void ) interrupt 0
{
	EZUSB_IRQ_CLEAR( );
	USBIRQ = bmSUTOK;      // Clear SUTOK IRQ
}

void ISR_Sof( void ) interrupt 0
{
	EZUSB_IRQ_CLEAR( );
	USBIRQ = bmSOF;        // Clear SOF IRQ
}

void ISR_Ures( void ) interrupt 0
{
	if ( EZUSB_HIGHSPEED( ) )
	{
		pConfigDscr = pHighSpeedConfigDscr;
		pOtherConfigDscr = pFullSpeedConfigDscr;
	}
	else
	{
		pConfigDscr = pFullSpeedConfigDscr;
		pOtherConfigDscr = pHighSpeedConfigDscr;
	}
   
	EZUSB_IRQ_CLEAR( );
	USBIRQ = bmURES;       // Clear URES IRQ
}

void ISR_Susp( void ) interrupt 0
{
	Sleep = TRUE;
	EZUSB_IRQ_CLEAR( );
	USBIRQ = bmSUSP;
}

void ISR_Highspeed( void ) interrupt 0
{
	if(EZUSB_HIGHSPEED())
	{
		pConfigDscr = pHighSpeedConfigDscr;
		pOtherConfigDscr = pFullSpeedConfigDscr;
	}
	else
	{
		pConfigDscr = pFullSpeedConfigDscr;
		pOtherConfigDscr = pHighSpeedConfigDscr;
	}

	EZUSB_IRQ_CLEAR();
	USBIRQ = bmHSGRANT;
}

void ISR_Ep0ack( void ) interrupt 0
{
}

void ISR_Stub( void ) interrupt 0
{
}

void ISR_Ep0in( void ) interrupt 0
{
}

void ISR_Ep0out( void ) interrupt 0
{
}

void ISR_Ep1in( void ) interrupt 0
{
}

void ISR_Ep1out( void ) interrupt 0
{
}

void ISR_Ep2inout( void ) interrupt 0
{
}

void ISR_Ep4inout( void ) interrupt 0
{
}

void ISR_Ep6inout( void ) interrupt 0
{
}

void ISR_Ep8inout( void ) interrupt 0
{
}

void ISR_Ibn( void ) interrupt 0
{
}

void ISR_Ep0pingnak( void ) interrupt 0
{
}

void ISR_Ep1pingnak( void ) interrupt 0
{
}

void ISR_Ep2pingnak( void ) interrupt 0
{
}

void ISR_Ep4pingnak( void ) interrupt 0
{
}

void ISR_Ep6pingnak( void ) interrupt 0
{
}

void ISR_Ep8pingnak( void ) interrupt 0
{
}

void ISR_Errorlimit( void ) interrupt 0
{
}

void ISR_Ep2piderror( void ) interrupt 0
{
}

void ISR_Ep4piderror( void ) interrupt 0
{
}

void ISR_Ep6piderror( void ) interrupt 0
{
}

void ISR_Ep8piderror( void ) interrupt 0
{
}

void ISR_Ep2pflag( void ) interrupt 0
{
}

void ISR_Ep4pflag( void ) interrupt 0
{
}

void ISR_Ep6pflag( void ) interrupt 0
{
}

void ISR_Ep8pflag( void ) interrupt 0
{
}

void ISR_Ep2eflag( void ) interrupt 0
{
}

void ISR_Ep4eflag( void ) interrupt 0
{
}

void ISR_Ep6eflag( void ) interrupt 0
{
}

void ISR_Ep8eflag( void ) interrupt 0
{
}

void ISR_Ep2fflag( void ) interrupt 0
{
}

void ISR_Ep4fflag( void ) interrupt 0
{
}

void ISR_Ep6fflag( void ) interrupt 0
{
}

void ISR_Ep8fflag( void ) interrupt 0
{
}

void ISR_GpifComplete( void ) interrupt 0
{
}

void ISR_GpifWaveform( void ) interrupt 0
{
}
